using Model;
using Model.Serialization;
using Model.Stub;
using Newtonsoft.Json.Linq;
using NuGet.Frameworks;
using System;

namespace TestUnitaires
{

    public class TU_LINQ_XML_Serialization
    {
        [Theory]
        [InlineData("Rference")]
        public void TU_Methods(string? test)
        {
            Playlist p = new(test, "PlaceHolder", "place.holder");
            Album album = new(test, "place.holder", new Artist("test"), "PlaceHolder", "PlaceHolder");
            CustomTitle t = new(test, "test. mp3", "Banger", "path");
            Artist artist = new(test);
            InfoTitle info = new(test, "url.test", "infos", "desc", Genre.POP, 345);
            LinqXmlSerialization l = new(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Data"));
            Assert.NotEmpty(l.Albums);
            Assert.NotEmpty(l.Artists);
            Assert.NotEmpty(l.InfoTitles);
            Assert.Empty(l.Playlists);
            Assert.Empty(l.CustomTitles);
            l.AddPlaylist(p);
            l.AddAlbum(album);
            l.AddCustomTitle(t);
            l.AddArtist(artist);
            l.AddInfoTitle(info);
            Assert.Contains(t, l.CustomTitles);
            Assert.Contains(album, l.Albums);
            Assert.Contains(p, l.Playlists);
            Assert.Contains(info, l.InfoTitles);
            Assert.Contains(artist, l.Artists);
            l.RemovePlaylist(p);
            l.RemoveAlbum(album);
            l.RemoveCustomTitle(t);
            l.RemoveInfoTitle(info);
            Assert.DoesNotContain(t, l.CustomTitles);
            Assert.DoesNotContain(p, l.Playlists);
            Assert.DoesNotContain(album, l.Albums);
            Assert.DoesNotContain(info,l.InfoTitles);
            l.AddCustomTitle(t);
            l.AddPlaylist(p);
            l.AddAlbum(album);
            l.AddArtist(artist);
            l.AddInfoTitle(info);
            Assert.Equal(l.GetAlbums(), l.Albums);
            Assert.Equal(l.GetInfoTitles(), l.InfoTitles);
            Assert.Equal(l.GetCustomTitles(), l.CustomTitles);
            Assert.Equal(l.GetPlaylists(), l.Playlists);
            Assert.Equal(l.GetArtists(), l.Artists);
            Assert.Equal(l.GetAlbumByName(test), album);
            Assert.Equal(l.GetPlaylistByName(test), p);
            Assert.Equal(l.GetInfoTitleByName(test), info);
            Assert.Equal(l.GetArtistByName(test), artist);
        }

        [Theory]
        [InlineData("Imagine Dragons")]
        [InlineData("Nepal")]
        [InlineData("Booba")]
        public void TU_ExistsArtist_Method(string name)
        {
            LinqXmlSerialization l = new(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Data"));
            Assert.True(l.ExistsArtist(new Artist(name)));
            Assert.True(l.ExistsArtistByName(name));
        }

        [Theory]
        [InlineData("Fenetre Sur Rue", "album2.jpg", "Hugo TSR", "Un banger", "Sortie : 2012")]
        [InlineData("Adios Bahamas", "album1.jpg", "Nepal", "Album post-mortem qui sign galement le dernier de l'artiste", "Sortie : 2020")]
        public void TU_ExistsAlbum_Method(string name, string imageURL, string artistName, string desc, string info)
        {
            LinqXmlSerialization l = new(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Data"));
            Assert.True(l.ExistsAlbum(new Album(name, imageURL, new Artist(artistName), desc, info)));
            Assert.True(l.ExistsAlbumByName(name));
        }

        [Theory]
        [InlineData("Trajectoire", "morceau1.png", "Sortie : 2020", "Morceau de Npal", Genre.HIP_HOP, 0)]
        [InlineData("Bones", "album14.jpg", "Titre de l'album Mercury Act 2", "La chanson \"Bones\" a t publie en tant que premier single de Mercury - Act 2 le 11 mars 2022. La chanson a t utilise pour promouvoir la troisime saison de la srie Amazon Prime Video The Boys. La sortie du clip de la chanson le 6 avril a concid avec la prcommande de l'album.", Genre.POP, 13)]
        public void TU_ExistsInfoTitles_Method(string name, string imageURL, string desc, string info, Genre genre, long id)
        {
            LinqXmlSerialization l = new(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Data"));
            Assert.True(l.ExistsInfoTitle(new InfoTitle(name, imageURL, desc, info, genre, id)));
            Assert.True(l.ExistsInfoTitleByName(name));
        }

        [Theory]
        [InlineData("Trajectoire", "morceau1.png", "Sortie : 2020")]
        [InlineData("Bones", "album14.jpg", "Titre de l'album Mercury Act 2")]
        public void TU_ExistsPlaylist_Method(string name, string imageURL, string desc)
        {
            LinqXmlSerialization l = new(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Data"));
            Assert.False(l.ExistsPlaylist(new Playlist(name, desc, imageURL)));
            Assert.False(l.ExistsPlaylistByName(name));
        }

        [Theory]
        [InlineData("Trajectoire", "morceau1.png", "Sortie : 2020", "path.mp3")]
        [InlineData("Bones", "album14.jpg", "Titre de l'album Mercury Act 2", "path.mp3")]
        public void TU_ExistsCustomTitle_Method(string name, string imageURL, string info, string path)
        {
            LinqXmlSerialization l = new(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Data"));
            Assert.False(l.ExistsCustomTitle(new CustomTitle(name, imageURL, info, path)));
            Assert.False(l.ExistsCustomTitleByName(name));
        }

        [Fact]
        public void TU_RemoveAddMultiple()
        {
            List<Album> albums = new()
            {
                new Album("album1", "img1.png", new Artist("artist"), "desc", "info"),
                new Album("album2", "img2.png", new Artist("artist"), "desc", "info"),
                new Album("album3", "img3.png", new Artist("artist"), "desc", "info")
            };
            List<Artist> artists = new()
            {
                new Artist("artist1"),
                new Artist("artist2"),
                new Artist("artist3")
            };
            List<InfoTitle> infoTitles = new()
            {
                new InfoTitle("it1", "img1.png", "desc", "info", Genre.POP, 0),
                new InfoTitle("it2", "img2.png", "desc", "info", Genre.POP, 0),
                new InfoTitle("it3", "img3.png", "desc", "info", Genre.POP, 0)
            };
            List<CustomTitle> customTitles = new()
            {
                new CustomTitle("ct1", "img1.png", "info", "path1.mp3"),
                new CustomTitle("ct2", "img2.png", "info", "path2.mp3"),
                new CustomTitle("ct3", "img3.png", "info", "path3.mp3")
            };
            List<Playlist> playlists = new()
            {
                new Playlist("nom1", "desc", "img1.png"),
                new Playlist("nom2", "desc", "img2.png"),
                new Playlist("nom3", "desc", "img3.png")
            };
            LinqXmlSerialization l = new(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Data"));
            l.AddAlbums(albums);
            l.AddArtists(artists);
            l.AddInfoTitles(infoTitles);
            l.AddCustomTitles(customTitles);
            l.AddPlaylists(playlists);

            l.RemoveAlbums(albums);
            l.RemoveArtists(artists);
            l.RemoveInfoTitles(infoTitles);
            l.RemoveCustomTitles(customTitles);
            l.RemovePlaylists(playlists);

            l.AddAlbums(albums);
            l.AddArtists(artists);
            l.AddInfoTitles(infoTitles);
            l.AddCustomTitles(customTitles);
            l.AddPlaylists(playlists);
        }

        [Theory]
        [InlineData("nom", "url2.png", "artist", "desc", "info", Genre.POP, "path.mp3")]
        [InlineData("nom", "url2.png", "", "desc", "info", Genre.POP, "path.mp3")]
        [InlineData("nom", "url2.png", "artist", "", "info", Genre.POP, "path.mp3")]
        [InlineData("nom", "url2.png", "artist", "desc", "", Genre.POP, "path.mp3")]
        [InlineData("nom", "url2.png", "artist", "desc", "", Genre.POP, "")]
        public void TU_Update(string nom, string url, string artistName, string description, string info, Genre genre, string path)
        {
            LinqXmlSerialization l = new(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Data"));

            Album album = new("album1", "img1.png", new Artist("artist1"), "desc", "info");
            Album album2 = new("album2", "img2.png", new Artist("artist1"), "desc", "info");

            Artist artist = new("artist1");
            Artist artist2 = new("artist2");

            InfoTitle infoTitle = new("it1", "img1.png", "desc", "info", Genre.RAP, 0);
            InfoTitle infoTitle2 = new("it12", "img12.png", "desc", "info", Genre.RAP, 0);

            CustomTitle customTitle = new("nom1", "img1.png", "info", "path1.mp3");
            CustomTitle customTitle2 = new("nom2", "img2.png", "info", "path2.mp3");

            Playlist playlist = new("nom1", "desc", "img1.png");
            Playlist playlist2 = new("nom2", "desc", "img2.png");


            l.AddAlbum(album);
            l.AddAlbum(album2);
            l.AddArtist(artist);
            l.AddArtist(artist2);
            l.AddInfoTitle(infoTitle);
            l.AddInfoTitle(infoTitle2);
            l.AddCustomTitle(customTitle);
            l.AddCustomTitle(customTitle2);
            l.AddPlaylist(playlist);
            l.AddPlaylist(playlist2);

            l.UpdateAlbum(album, nom, url, new Artist(artistName), description, info);
            l.UpdateAlbumByName(album2.Name, nom, url, new Artist(artistName), description, info);

            l.UpdateArtist(artist, nom);
            l.UpdateArtistByName(artist2.Name, nom);

            l.UpdateInfoTitle(infoTitle, nom, url, info, new Artist(artistName), description, genre);
            l.UpdateInfoTitleByName(infoTitle2.Name, nom, url, info, new Artist(artistName), description, genre);

            l.UpdateCustomTitle(customTitle, nom, url, info, path);
            l.UpdateCustomTitleByPath(customTitle2.Path, nom, url, info, path);

            l.UpdatePlaylist(playlist, nom, description, url);
            l.UpdatePlaylistByName(playlist2.Name, nom, description, url);

            List<Album> albums = new()
            {
                album, album2
            };

            List<Artist> artists = new()
            {
                artist, artist2
            };
            List<InfoTitle> infoTitles = new()
            {
                infoTitle, infoTitle2
            };
            List<CustomTitle> customTitles = new()
            {
                customTitle, customTitle2
            };
            List<Playlist> playlists = new()
            {
                playlist, playlist2
            };

            foreach(Album a in albums)
            {
                Assert.Equal(nom, a.Name);
                Assert.Equal(url, a.ImageURL);
                Assert.Equal(artistName, a.Artist.Name);
                Assert.Equal(description, a.Description);
                Assert.Equal(info, a.Information);
            }
            foreach(Artist a in artists)
            {
                Assert.Equal(nom, a.Name);
            }
            foreach(InfoTitle it in infoTitles)
            {
                Assert.Equal(nom, it.Name);
                Assert.Equal(url, it.ImageURL);
                Assert.Equal(description, it.Description);
                Assert.Equal(info, it.Information);
                Assert.Equal(genre, it.Genre);
            }
            foreach(CustomTitle ct in customTitles)
            {
                Assert.Equal(nom, ct.Name);
                Assert.Equal(url, ct.ImageURL);
                Assert.Equal(info, ct.Information);
                Assert.Equal(path, ct.Path);
            }
            foreach(Playlist p in playlists)
            {
                Assert.Equal(nom, p.Name);
                Assert.Equal(description, p.Description);
                Assert.Equal(url, p.ImageURL);
            }
        }

        [Theory]
        [InlineData(Genre.ROCK, "ROCK")]
        [InlineData(Genre.POP, "POP")]
        [InlineData(Genre.K_POP, "K_POP")]
        [InlineData(Genre.HIP_HOP, "HIP_HOP")]
        [InlineData(Genre.CLASSIQUE, "CLASSIQUE")]
        [InlineData(Genre.VARIETE_FRANCAISE, "VARIETE_FRANCAISE")]
        [InlineData(Genre.VARIETE_INTERNATIONALE, "VARIETE_INTERNATIONALE")]
        [InlineData(Genre.RAP, "RAP")]
        [InlineData(Genre.BLUES, "BLUES")]
        [InlineData(Genre.ELECTRO, "ELECTRO")]
        [InlineData(Genre.COUNTRY, "COUNTRY")]
        [InlineData(Genre.DISCO, "DISCO")]
        [InlineData(Genre.METAL, "METAL")]
        [InlineData(Genre.FUNK, "FUNK")]
        [InlineData(Genre.GOSPEL, "GOSPEL")]
        [InlineData(Genre.RNB, "RNB")]
        [InlineData(Genre.REGGAE, "REGGAE")]
        [InlineData(Genre.JAZZ, "JAZZ")]
        public void TU_Genre(Genre genre, string genreString)
        {
            Assert.Equal(genre, LinqXmlSerialization.GetGenreByName(genreString));
        }

        [Fact]
        public void TU_SaveLoad()
        {
            LinqXmlSerialization l = new(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Data"));
            LinqXmlSerialization.SaveAlbums();
            LinqXmlSerialization.SaveInfoTitles();
            LinqXmlSerialization.SaveArtists();
            l.SavePlaylists();
            l.SaveCustomTitles();
            l.SaveSerialization();
            l.LoadSerialization();

            l.LoadAlbums();
            l.LoadArtists();
            l.LoadInfoTitles();
            l.LoadCustomTitles();
            l.LoadPlaylists();
            l.LoadSerialization();

            Assert.NotEmpty(l.Albums);
            Assert.NotEmpty(l.Artists);
            Assert.NotEmpty(l.InfoTitles);
            Assert.Empty(l.CustomTitles);
            Assert.Empty(l.Playlists);

        }
    }
}